#include <stdio.h>
#include <ctype.h>
#define MAX_SEQ_LEN 1000000
int ted_nnl_file_to_vec();
int file_to_vec();


/*   Looks for a sequence with two or fewer mismatches in
the first 100 bases of an input sequence.  It first looks for
an exact match, then backs off and allows one mismatch, then
backs off one more time and allows two mismatches.  If it
does not find it, it still outputs a 0.

  I took pieces from my reformat.c and ted/seqIOEdit.c  to make a
clip_left_vec.c which just looks for a GATCCCC in the same way ted looks
for it.  Right now it's usage would be:

        clip_left_vec sequence_file_name enzyme

and it prints the left cutoff to standard out.
*/



main(argc,argv)
int argc;
char **argv;
{ char seq[MAX_SEQ_LEN];
  char seq_fn[100];
  char f_name[100];
  char fof_fn[100];
  int seq_len;
  char str_seqlen[110];
  int alpha;
  int ted_okay;
  int i,j,k,kk,file_okay;
  FILE *fp,*gp;
  char seqlen_name[500];
  char enz[100];  
  char tempenz[100];

  
  /* get the sequence filename */
  if (argc>=2)
    strcpy(seq_fn, argv[1]);
  else {
    fprintf(stdout,"sequence filename?");
    scanf("%s",seq_fn);
  }


  if (argc>=3)
    strcpy(tempenz, argv[2]);
  else {
    fprintf(stdout,"bases to search for?");
    scanf("%s",tempenz);
  }

  for (i=0; i<strlen(tempenz); i++) {
    enz[i]=tolower(tempenz[i]);
  }

  
  
  /*read in the sequence file, getting rid of any numbers
    at the start of the line, or any a greater
    than sign  as comments
    or a ; at the start indicating its a ted file;
    convert it to upper case*/
  ted_okay = ted_nnl_file_to_vec(seq_fn,seq,MAX_SEQ_LEN);
  if (ted_okay!=0) {
    seq_len=strlen(seq);
  } /* if ted_okay*/
  else {
    fprintf(stdout,"%s did not contain any sequence\n",seq_fn);
    exit();
  }
  
  printf("%d\n",findLeftCutoff(seq, enz) );
  
} 


/* variables relating to the opening of ted windows */
int tedNedBases;
int tedleftCutoff;
int tedSeqLen; /* NedBases-rightCutoff-leftCutoff */
char tedType[4]; /*ABI, PLN or ALF */
char tedFileName[200]; /* usually just the sequence filename less the .seq
 */

int ted_nnl_file_to_vec(f_name,vec,max_n)
char *f_name;
char *vec;
int max_n;
/* int ted_nnl_file_to_vec(f_name, vec, max_n)
	input: char *f_name; int max_n;
	output: char *vec;
	looks at the first char for a semicolon, if it is,
	it knows the file has been input from ted, so it
	gets the information about the sequence from the 
	first line and gets the sequence from the rest 
	
	reads the input file, f_name, to a vec array of chars
 with a maximum size of max_n; does not read in any of
the end of line characters , returns the size of vec array.*/

{
    FILE *fopen(), *fp;
    int i,n,j;
    int zero=0;
    char line[500];
    int lookBlank,lookBlank_i,newLine;
    int MAXLINE=500;
    int comment=0; /* keeps track of whether there was 
		    a comment line or not */
    int more_comments=0;


    lookBlank=0;
    newLine=1;
    


    if (NULL == (fp = fopen(f_name, "r"))) {
      return(0);
    }

    /* go read the first line of the sequence to see if it is
       a ted file or a comment line */
    fgets(line, MAXLINE, fp); /* fgets only reads a line at a time*/

    if (line[0]==';')  {
    sscanf(line,";%6d%6d%6d%s%s\n",&tedNedBases,&tedleftCutoff,&tedSeqLen,tedType,tedFileName);
    comment=1;
  }
    else { /* I check to make sure that tedType[0] is either A or P for ABI or PLN
	      before I will actually try to run a ted window.  So if I blank out
	      the value for tedType when the file wasn't ted file,  then the
	      program will not be tempted to open a ted window on a non-ted
	      file */
      tedType[0]='\0';

    /* also check to see if the first line is a comment, by checking
     for greater than signs */
      comment=0;
      for (i=0; line[i]; i++) {

	if (line[i]=='>') comment=1;
	/* then this must be a comment line, just
	   get the rest of the line and throw it away*/


      }

    }

    
    if (comment==1 || tedType[0]!='\0') more_comments=1;
    /* if there were any comments go past any more lines that were comments */
      /* fgets only reads a line at a time, returns a NULL when eof*/
    while (fgets(line, MAXLINE, fp)!=NULL && more_comments==1)  {
      for (i=0; line[i]; i++) {
	if (line[i]=='>' || line[i]==';') { comment++; break;}
	/* then this must be a comment line, just
	   get the rest of the line and throw it away*/
	more_comments=0;
      }
    }
    

    /* close the file because well start over reading it again after
       we check the first line */
    fclose(fp);


    if (NULL == (fp = fopen(f_name, "r"))) {
      return(0);
    }

    /* if it was a comment or ted file find the end of the first line*/
    if (comment>0) {
      /* then find the end of the first line, the comment */
      for (j=0; j<comment; j++) 
	if (fgets(line, MAXLINE, fp)==NULL) break;
    }





/* all the lookBlank and new line stuff is looking for any
   groups of numbers starting a line if it finds after a 
   newline a set of contiguous numbers then a blank before it finds an
   alpha it assumes those were just nucleotide position
   numbers*/

    for (i = 0; i < max_n; i++) { 
      vec[i] = getc(fp);
      /* make sure you leave the check for vec[i] being EOF 
	 as the first check*/
      if (vec[i] == EOF) {
	vec[i] = '\0';
	return(i);
      }

      /*check if the sequence is numbered on its left end side*/
      if (isdigit(vec[i]) && newLine==1) {
	/* keep looking to see if you find a blank before any more alphas,
	 which would indicate that there are numbers in the sequence */
	lookBlank=1;
	lookBlank_i = i;
	newLine=0;
      }

      /* make sure that all '-' are 'N' and that
	all characters are lowercase */
      if (vec[i]=='-') vec[i]='n';
      else if (isalpha(vec[i])) {
	newLine=0;
	lookBlank=0;
	if (!islower(vec[i])) vec[i]=tolower(vec[i]);
      }
      /*throw away any new lines*/
      if (vec[i] == '\n') {
	if (i>-1) i--;
	lookBlank=0;
        newLine=1;
      }
      /*throw away any blanks*/
      if (vec[i] == ' ') {
	if (i>-1) i--;
	if (lookBlank==1) { /* go back and throw away all those
			       digits between the \n and the blank,
			       because they must be numbering of
			       the sequence*/
	   i=lookBlank_i-1;
	 }
      } /* if vec[i] is a blank*/

      if (!isalpha(vec[i]) && !isdigit(vec[i]) && vec[i]!=' ' && i>-1) {
/*	printf("WARNING: Your sequence contains a non-alphabetic character, %c.\nThis character was removed from the sequence.\n",vec[i]);*/
	if (i>-1) i--;
      }

     }/* for i=0, i< max_n*/
  
    printf ("file_to_vec vector size exceeded when reading file %s\n",f_name);
    return(0);
   }
	

int file_to_vec(f_name,vec,vec_st,max_n)
char *f_name;
char *vec; 
int *vec_st;
int max_n; 

/* file_to_vec(f_name, vec, vec_st,max_n)
	input: char *f_name; int max_n;
	output: char *vec, int *vec_st;
	reads the input file, f_name, to a vec array of chars of
size vec_st with a maximum size of max_n 

returns an error message in str, if there is one. Returns a 0 if there
was some problem when reading in the file */

{
    FILE *fopen(), *fp;
    int i;

    if (NULL == (fp = fopen(f_name, "r"))) {
	printf("\nERROR: file %s does not exist.  See file_to_vec.c\n",f_name);
	return(0);
    }
    for (i = 0; i < max_n; i++) 
      if (EOF == (vec[i] = getc(fp))) {
	vec[++i] = '\0';
	*vec_st = i;
	return(1);
      }
    printf ("vector size exceeded when reading file %s\n",f_name);
    return(0);
}	
	


int findLeftCutoff(char *theSeq, char *enzString)
{
    int maxStartPos=100; /* if the enzyme site wasn't found before this
                            baseNum, then that's probably not the cloning
                            site */
    int i,j,found;
    /* int jj,kk; */
    int indices[100];
    int num_matches;
    int num_bases;
    /* char enztemp[100]; */

    int cut_point=0; 
    
    num_bases=strlen(theSeq);


    memset ((void *)indices, 0, (size_t) (sizeof(int)*100));

    for (i=0; i<3; i++) {
        num_matches=string_match(enzString,strlen(enzString),theSeq,num_bases,i,indices);
        if (num_matches > 0) 
            if (indices[0] < maxStartPos) {
                found = 1;
                break;
            }
    }



    if (found==1)
    cut_point = indices[0] + strlen(enzString) + 1;

    return(cut_point);
  }

int string_match(seq1, n1, seq2, n2, nmiss, indices)

/*
  This function may be called once from initialDisplayedSeq if
  the user has input a string they wish to search for in the
  input sequence -- this subroutine returns *indices, the first
  position at which the query sequence (app_resources.astring)
  matched the input file sequence (currSeq). */

/* a modified version of match.c 
  finds alignments between a search sequence, seq1, and a target sequence, 
  seq2, with no gaps (except possibly at ends) 
   and at most nmiss mismatches (relative to seq1),
   n1 is the length of seq1 where seq1 is assumed to start at indices 0,
   n2 is the length of seq2 where seq2 is assumed to start at indices 0 */
      char *seq1, *seq2;
      int n1, n2, nmiss;
      int *indices;
{
    int i, j, d, istart, iend, i_miss, n_match;
    int mtable[100][5];  /* 100 would be, now the total number
			    of matches the program may find between
			    a given input string and the sequence */


/* d = j - i is the "offset" between the two sequences */
    if (n1 - n2 > nmiss) return (0);
    n_match = 0;
    for (d = -nmiss; d <= n2 + nmiss - n1; d++){
	if (d < 0)  istart = i_miss = -d;
	else istart = i_miss = 0;
	if (d > n2 - n1) {
	    iend = n2 - d;
	    i_miss += n1 + d - n2;
	}
	else iend = n1;
	for (i = istart, j = d + i; i < iend; i++, j++)
	    if (seq1[i] != seq2[j] && ++i_miss > nmiss) goto nextd; 
        mtable[n_match][0] = d + istart; /* indices (assuming they start at
					    0) of starting nucleotide in the
					    searched sequence */
        mtable[n_match][1] = istart; /* nucleotide position in the query
					sequence where match starts (assuming
					query index starts with 0) */
        mtable[n_match][2] = iend - istart; /* number of nucleotides in the
					       match */
	if (mtable[n_match][2]==n1) {
	  indices[n_match] = mtable[n_match][0];
	}
	  
        mtable[n_match][3] = i_miss; /* number of mismatches */
	n_match++; /* number of matches */
	if (n_match >= 100) return(n_match);

    nextd:;
    }

    return (n_match);
    /*return(0);*/ /*return 0 if it got to this point; that means
		 it must have not found an exact match, so 
		 you want the baseNum to be 0 in that case*/
}

