/*  Last edited: Mar 21 13:34 1995 (badger) */
/*
 * 21-Mar-1995 fasta entry name should be that of file not ID line (may be different)
 */
#include <stdio.h>
#include <ctype.h>

#include "expFileIO.h"

char *tail(char *path)
{
    char *c;
    c = path;
    for(;*path;path++) {
	if(*path=='/')c=path+1;
    }
    return c;
}


static void determine_active_region(Exp_info *e, int *left, int *right, int *left_reason, int *right_reason)
{
    /*
     * need to determine active region
     */
    int SQlen;
    int CSfrom, CSto, SL, SR, QL, QR;
    int l,r;
    int why_l,why_r;
    int noCS; /* if there is no CS line */

    SQlen = strlen(exp_get_entry(e,EFLT_SQ));
    noCS = exp_get_rng(e,EFLT_CS,&CSfrom,&CSto);


    l = 0;
    why_l = -1;

    r = SQlen+1;
    why_l = -2;


    if (! exp_get_int(e,EFLT_QL,&QL)) {
	if (l<QL) {
	    l = QL;
	    why_l = EFLT_QL;
	}
    }
    if (! exp_get_int(e,EFLT_SL,&SL)) {
	if (l<SL) {
	    l = SL;
	    why_l = EFLT_SL;
	}
    }

    if (! exp_get_int(e,EFLT_QR,&QR)) {
	if (r>QR) {
	    r = QR;
	    why_r = EFLT_QR;
	}
    }
    if (! exp_get_int(e,EFLT_SR,&SR)) {
	if (r>SR) {
	    r = SR;
	    why_r = EFLT_SR;
	}
    }

    if (!noCS) {
        /*
         * Six posibilities:
         *           l                r
         * 1  -----  >                <
         * 2  -------|------->        <
         * 3  -------|<---------------|-------
         * 4         >    <--------   |
         * 5         >    <-----------|-------
         * 6         >                <  -----
         *
         * key: [-] CS [>] new left c.o [<] new right c.o [|] orig c.o
         *
         * Coded for understanding not efficiency!!!
         */
        if (/*1*/ CSfrom <= l && CSto <= l) {
            /* null statement */
        } else if (/*2*/ CSfrom <= l+1 && CSto < r) {
            l = CSto;
	    why_l = EFLT_CS;
        } else if (/*3*/ CSfrom <= l+1 && CSto >= r) {
            r = l+1;
	    why_r = EFLT_CS;
        } else if (/*4*/ CSfrom < r && CSto < r) {
            r = CSfrom;
	    why_r = EFLT_CS;
        } else if (/*5*/ CSfrom < r && CSto >= r) {
            r = CSfrom;
	    why_r = EFLT_CS;
        } else if (/*6*/ CSfrom >= r && CSto >= r) {
            /* null statement */
	}
    }
    if (l>r) l = r-1; /* just a quick consistency check */

    *left = l;
    *right = r;

    *left_reason = why_l;
    *right_reason = why_r;
}


int parse(Exp_info *e, char *arg, int *val, int *reason, int want_from)
{
    int left, right;
    int left_reason, right_reason;
    int err;

    /**val = 0; now has default */
    /**reason = -3; now has default */
    err = 0;
    if (isdigit(*arg)) {
	*val = atoi(arg);
	*reason = -3;
    } else if (strcmp(arg,"left")==0) {
	determine_active_region(e, &left, &right, &left_reason, &right_reason);
	if (right <= left + 1) {
	    err = 1;
	} else {
	    *val = left + 1;
	    *reason = left_reason;
	}
    } else if (strcmp(arg,"right")==0) {
	determine_active_region(e, &left, &right, &left_reason, &right_reason);
	if (right <= left + 1) {
	    err = 1;
	} else {
	    *val = right - 1;
	    *reason = right_reason;
	}
    } else {
	int entry;
	while(*arg) {
	    entry = -1;
	    switch(*arg) {
	    case 'q' : case 'Q' :
		arg++;
		switch(*arg) {
		case 'l' : case 'L' : arg++; entry = EFLT_QL; break;
		case 'r' : case 'R' : arg++; entry = EFLT_QR; break;
		default: arg++; break;
		}
		if (*arg == ',') arg++;
		break;
	    case 's' : case 'S' :
		arg++;
		switch(*arg) {
		case 'l' : case 'L' : arg++; entry = EFLT_SL; break;
		case 'r' : case 'R' : arg++; entry = EFLT_SR; break;
		default: arg++; break;
		}
		if (*arg == ',') arg++;
		break;
	    default: arg++; break;
	    }

	    if (entry == -1) {
		fprintf(stderr,"Unparsable sequence coordinate.\n");
		while(*arg && *arg!=',') arg++;
	    } else {
		int new_val;
	        if (0==exp_get_int(e,entry,&new_val)) {
		    /* adjust values according to end we want */
		    if (want_from) {
			if (entry==EFLT_SL || entry==EFLT_QL) new_val++;
			if (new_val > *val)  { *val = new_val; *reason=entry; }
		    } else {
			if (entry==EFLT_SR || entry==EFLT_QR) new_val--;
			if (new_val < *val) { *val = new_val;; *reason=entry; }
		    }
		    
		}
	    }
	}
    }
    return err;
}



void print_reason(int pos, int reason)
{
    switch(reason) {
    case -2: printf("end"); break;
    case -1: printf("start"); break;
    case EFLT_QL: printf("QL"); break;
    case EFLT_QR: printf("QR"); break;
    case EFLT_SL: printf("SL"); break;
    case EFLT_SR: printf("SR"); break;
    case EFLT_CS: printf("CS"); break;
    default: printf("%d",pos); break;
    }
}

static
void
usage()
{
    fprintf(stderr,"Usage: exp-piece [-p | -s | -q | -name text] from to [file]\n");
    fprintf(stderr,"Option     Description\n");
    fprintf(stderr,"-p         Phrap mode. Same as -s -q\n");
    fprintf(stderr,"-s         Mask sequencing vector\n");
    fprintf(stderr,"-q         Show low quality regions as lower case\n");
    fprintf(stderr,"-name text Use text as fasta entry name\n");
    fprintf(stderr,"\n");
    fprintf(stderr,"From/To    Description\n");
    fprintf(stderr,"int        By base position\n");
    fprintf(stderr,"0          End of sequence\n");
    fprintf(stderr,"-int       Bases from end of sequence\n");
    fprintf(stderr,"left       First useful base\n");
    fprintf(stderr,"right      Last useful base\n");
    fprintf(stderr,"tag{,tag}  From cutoff (SL,QL,SR,QR)\n");
    fprintf(stderr,"\n");
}

int main(int argc, char *argv[])
{
    Exp_info *e;
    char *name = NULL;
    int i,j,k; /*loop variables*/
    int from, from_reason;
    int to, to_reason;
    int len; /*Seq len*/
    int err;			/* error from parse() */
    char *seq;
    int SQlen, QL, QR, SL, SR; 
    int qual = 0;
    int svec = 0;


    /* shift */
    argc--; argv++;

    while (argc && argv[0][0] == '-') {

	if (strcmp("-p",argv[0]) == 0) {
	    qual=1;			/* turn on quality fix */
	    svec=1;			/* turn on vector fix */
	} else if (strcmp("-q",argv[0]) == 0) {
	    qual=1;			/* turn on quality fix */
	} else if (strcmp("-s",argv[0]) == 0) {
	    svec=1;			/* turn on vector fix */
	} else if (strcmp("-name",argv[0]) == 0) {
	    argc--; argv++;
	    if (argc) {
		name = argv[0];
	    } else { 
		fprintf(stderr,"exp-piece: -name argument expected\n");
		usage(); return 2;
	    }
	} else {
	    fprintf(stderr,"exp-piece: unknown command line flag\n");
	    usage();
	    return(2);
	}
	/* shift */
	argc--; argv++;
    }


    switch(argc) {
/* routine exp_fread_info() not implemented yet
    case 2:
        if ( (e = exp_fread_info(stdin)) == NULL ) return 1;
	break;
*/
    case 3:
        if ( (e = exp_read_info(argv[2])) == NULL ) return 1;
	break;
    default: usage(); return 2;

    }

    if ( !exp_Nentries(e,EFLT_SQ)) return 1;

    /* Print out header */
    if (name) {
	    printf(">%s",tail(name));
    } else if (argc==4) {
	    /* tail */
	    printf(">%s",tail(argv[2]));
    } else if (exp_Nentries(e,EFLT_ID)) {
	    printf(">%s",exp_get_entry(e,EFLT_ID));
    } else {
	    printf(">");
    }

    /* parse command line arguments */
    seq = exp_get_entry(e,EFLT_SQ);
    len = strlen(seq);
    from = 1; to = len;
    from_reason = -1; to_reason = -2;
    err = parse(e,argv[0],&from, &from_reason,1/*from*/) + parse(e,argv[1],&to,&to_reason,0/*to*/);

    /* Check the values */
    /*if (from > len) from = len;*/
    if (to <= 0) to = len + to;
    /*if (to < from) to = from - 1;*/

    if (from == 1) from_reason = -1;
    if (to == len) to_reason = -2;

    if (err) {
        printf("  sequence completely clipped");
    } else {
        printf("  bases %d to %d",from,to);


	printf(" (");
	print_reason(from,from_reason);
	printf(" to ");
	print_reason(to,to_reason);
	printf(")");
    }

    if ( svec || qual ) {
      
      /* Re-format sequence if pg option specified:
	 
	 from base 1 to base QL in lowercase,
	 from base QL to end in lowercase,
	 from SL to SR convert to 'X'          */
      
      if (exp_get_int(e,EFLT_SL,&SL)) SL = 0;
      if (exp_get_int(e,EFLT_SR,&SR)) SR = len+1;
      if (exp_get_int(e,EFLT_QL,&QL)) QL = 0;
      if (exp_get_int(e,EFLT_QR,&QR)) QR = len+1;
      
      /*fprintf (stderr, "len: (%d) QL: (%d)  QR: (%d)  SL: (%d)  SR: (%d)\n", len, QL, QR, SL, SR);*/
      
      for(k=from;k<=to;k++) {
	
	seq[k-1]=tolower(seq[k-1]);

	if ( qual && (k >= QL) && (k <= QR) ) {
	  seq[k-1]=toupper(seq[k-1]);
	}
	
	if ( svec && (k<=SL) || (k>=SR) ) {
	  seq[k-1]='X';
	}
      }
    }
	  
    /* Print out sequence */
    if (err == 0 &&
	from <= len && from > 0 &&
	to <= len && to > 0 &&
	from <= to) {
      for(j=0,i=from-1;i<to;j++,i++) {
	if (j%60==0) printf("\n");
	printf("%c",seq[i]);
      }
    }
    printf("\n");


    /* Shut this baby down! */
    exp_destroy_info(e);

    return 0;
}
